=for comment $Id: pod_postamble.tpl,v 1.1 2005/06/26 16:41:00 robertemay Exp $

=head1 VERSION

Documentation for Win32::GUI v__W32G_VERSION__ created __W32G_DATE__

This document is autogenerated by the build process. Edits made here will be lost.
Edit F<__W32G_FILE__> instead.

=head1 SUPPORT

Homepage: L<__W32G_WEB_HOMEPAGE__>.

For further support join the users mailing list(__W32G_EMAIL_USERLIST__) from the website
at L<__W32G_WEB_USERMAIL__>.  There is a searchable list archive at L<__W32G_WEB_MAILARCHIVE__>.

=head1 COPYRIGHT and LICENCE

Copyright (c) 1997..__W32G_YEAR__ Aldo Calpini. All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

