=for comment $Id: per_package_method_section.tpl,v 1.1 2005/06/26 16:41:00 robertemay Exp $

=head1 METHODS

L<Common methods|Win32::GUI::Reference::Methods> apply to most windows, controls and resources.

__W32G_PP_METHODS__
