=for comment $Id: per_package_event_section.tpl,v 1.1 2005/06/26 16:41:00 robertemay Exp $

=head1 EVENTS

L<Common events|Win32::GUI::Reference::Events> apply to most windows and controls.

__W32G_PP_EVENTS__

